package com.pruebasaplicacion.sergimeli.bletests;

import android.bluetooth.BluetoothGattCharacteristic;

public class SensorTagData {

    /**
     * Método que convierte los bytes leídos de la característica en parámetros que se pueden
     * almacenar en un array de float
     * @param c característica a leer
     * @return array float con datos del gyroscopio
     */
    public static float[] extractGyroData(BluetoothGattCharacteristic c){
        //byte[] value = c.getValue();
        float gyroData[] = new float[3];
        gyroData[0] = shortSignedAtOffset(c, 0) * (500f / 65536f) * -1;
        gyroData[1] = shortSignedAtOffset(c, 2) * (500f / 65536f);
        gyroData[2] = shortSignedAtOffset(c, 4) * (500f / 65536f);

        return gyroData;

    }

    /**
     * Método que convierte los bytes leídos de la característica en parámetros que se pueden
     * almacenar en un array de float
     * @param value byte leído de la característica del acelerómetro
     * @return array con todos los valores float
     */
    public static float[] extractAccData(byte[] value){

        final float SCALE = (float) 4096.0;
        float[] Accsent = new float[4];

        Accsent[0] = (value[0]<<8)/SCALE;
        Accsent[1] = (value[1]<<8)/SCALE;
        Accsent[2] = (value[2]<<8)/SCALE;

        return Accsent;
    }

    /**
     * Estos métodos devuelven un offset dependiendo del formato a utilizar en los previos
     * metodos.
     * @param c característica a leer
     * @param offset offset requerido
     * @return Byte cambiado según el offset necesário
     */
    private static Integer shortSignedAtOffset(BluetoothGattCharacteristic c, int offset) {
        Integer lowerByte = c.getIntValue(BluetoothGattCharacteristic.FORMAT_UINT8, offset);
        Integer upperByte = c.getIntValue(BluetoothGattCharacteristic.FORMAT_SINT8, offset + 1); // Note: interpret MSB as signed.

        return (upperByte << 8) + lowerByte;
    }

    private static Integer shortUnsignedAtOffset(BluetoothGattCharacteristic c, int offset) {
        Integer lowerByte = c.getIntValue(BluetoothGattCharacteristic.FORMAT_UINT8, offset);
        Integer upperByte = c.getIntValue(BluetoothGattCharacteristic.FORMAT_UINT8, offset + 1); // Note: interpret MSB as unsigned.

        return (upperByte << 8) + lowerByte;
    }
}