package com.pruebasaplicacion.sergimeli.bletests.sqlite;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteQueryBuilder;

import com.pruebasaplicacion.sergimeli.bletests.modelo.DatosDispositivo;
import com.pruebasaplicacion.sergimeli.bletests.modelo.RendimientoUsuario;
import com.pruebasaplicacion.sergimeli.bletests.modelo.Usuario;
import com.pruebasaplicacion.sergimeli.bletests.sqlite.BaseDatosDispositivo.Tablas;

/**
 * Created by Elerionfly on 22/07/2016.
 */
public class OperacionesBaseDatos {

    private static BaseDatosDispositivo baseDatos;
    private static OperacionesBaseDatos instancia = new OperacionesBaseDatos();

    private static final String DATOS_DISPOSITIVO_JOIN_USUARIO =
            "usuario INNER JOIN datos_dispositivo "+
                    "ON usuario.Usuario_Device = datos_dispositivo.mDevice";
    private static final String USUARIO_JOIN_RENDIMIENTO_USUARIO =
            "rendimiento_usuario INNER JOIN usuario "+
                    "ON rendimiento_usuario.Nombre = usuario.Nombre";

    private final String[] proyUsuarios = new String[]{
            Tablas.DATOS_USUARIO + "." + DeviceContract.Usuarios.ID,
            Tablas.DATOS_USUARIO + "." + DeviceContract.Usuarios.NOMBRE,
            Tablas.DATOS_USUARIO + "." + DeviceContract.Usuarios.ALTURA,
            Tablas.DATOS_USUARIO + "." + DeviceContract.Usuarios.PESO,
            Tablas.DATOS_USUARIO + "." + DeviceContract.Usuarios.ZANCADA,
            Tablas.DATOS_USUARIO + "." + DeviceContract.Usuarios.LASTLOG,
            Tablas.DATOS_USUARIO + "." + DeviceContract.Usuarios.RETODIARIO,
            Tablas.DATOS_USUARIO + "." + DeviceContract.Usuarios.RETOCOMPLTADO,
            Tablas.DATOS_DISPOSITIVO + "." + DeviceContract.DatosDispositivos.MSERVICEUUID,
            Tablas.DATOS_DISPOSITIVO + "." + DeviceContract.DatosDispositivos.MDEVICENAME,
            Tablas.DATOS_DISPOSITIVO + "." + DeviceContract.DatosDispositivos.GIVENNAME};

    private final String[] proyRendimientoUsuario = new String[]{
            Tablas.DATOS_RENDIMIENTO_USUARIO + "." + DeviceContract.RendimientoUsuarios.ID,
            Tablas.DATOS_USUARIO + "." + DeviceContract.Usuarios.NOMBRE,
            Tablas.DATOS_RENDIMIENTO_USUARIO + "." + DeviceContract.RendimientoUsuarios.FECHA,
            Tablas.DATOS_RENDIMIENTO_USUARIO + "." + DeviceContract.RendimientoUsuarios.PASOS,
            Tablas.DATOS_RENDIMIENTO_USUARIO + "." + DeviceContract.RendimientoUsuarios.PASOS_MALOS,
            Tablas.DATOS_RENDIMIENTO_USUARIO + "." + DeviceContract.RendimientoUsuarios.PASOS_MALOS_2,
            Tablas.DATOS_RENDIMIENTO_USUARIO + "." + DeviceContract.RendimientoUsuarios.PASOS_PESADOS,
            Tablas.DATOS_RENDIMIENTO_USUARIO + "." + DeviceContract.RendimientoUsuarios.DISTANCIA,
            Tablas.DATOS_RENDIMIENTO_USUARIO + "." + DeviceContract.RendimientoUsuarios.CALORIAS_QUEMADAS
            };

    private OperacionesBaseDatos(){
    }

    public static OperacionesBaseDatos obtenerInstancia(Context context){
        if(baseDatos == null){
            baseDatos = new BaseDatosDispositivo(context);
        }

        return instancia;
    }

    /**
     * [OPERACIONES_DATOS_DISPOSITIVO]
     * El siguiente método obtendrá todos los datos de un dispostivo.
     */

    public Cursor obtenerDatosDispositivo(){
        SQLiteDatabase db = baseDatos.getReadableDatabase();
        String sql = String.format("SELECT * FROM %s", Tablas.DATOS_DISPOSITIVO);
        return db.rawQuery(sql, null);
    }

    /**
     * El siguiente método obtendrá los datos de un dispositivo seleccionado
     * mediante ID.
     */

    public Cursor obtenerDatosPorIdDispositivo(String idDispositivo){
        SQLiteDatabase db = baseDatos.getReadableDatabase();
        String sql = String.format("SELECT * FROM %s WHERE %s=?",
                Tablas.DATOS_DISPOSITIVO, DeviceContract.DatosDispositivos.MDEVICE);
        String[] selectionArgs = {idDispositivo};

        return db.rawQuery(sql, selectionArgs);
    }

    /**
     * El siguiente método inserta Datos sobre el Dispositivo a través de una
     * instancia DatosDispositivo. El retorno es la llave compuesta del nuevo
     * detalle de forma: 'idDispositivo#mDevice'.
     */

    public String insertarDatosDispositivo(DatosDispositivo datos){
        SQLiteDatabase db = baseDatos.getWritableDatabase();

        ContentValues values = new ContentValues();
        values.put(DeviceContract.DatosDispositivos.MDEVICE, datos.mDevice);
        values.put(DeviceContract.DatosDispositivos.MSERVICEUUID, datos.mServiceUUID);
        values.put(DeviceContract.DatosDispositivos.MDEVICENAME, datos.mDeviceName);
        values.put(DeviceContract.DatosDispositivos.GIVENNAME, datos.GivenName);

        db.insertOrThrow(Tablas.DATOS_DISPOSITIVO, null, values);

        return datos.mDevice;
    }

    /**
     * Se actualizan los datos a través de nuevos valores que vienen en una instancia
     * DatosDispositivo. Esta solo se lleva a cabo si el registro tiene el mismo id que los datos y
     * mismo mDevice.
     */

    public boolean actualizarNombreDispositivo(DatosDispositivo datos){
        SQLiteDatabase db = baseDatos.getWritableDatabase();

        ContentValues values = new ContentValues();
        values.put(DeviceContract.DatosDispositivos.GIVENNAME, datos.GivenName);

        String selection = String.format("%s=?",
                DeviceContract.DatosDispositivos.MDEVICE);
        final String[] whereArgs = {datos.mDevice};

        int resultado = db.update(Tablas.DATOS_DISPOSITIVO, values, selection, whereArgs);
        return resultado > 0;
    }

    /**
     * Se elimina un registro de la tabla Datos_Dispositivo cuyo  mDevice concuerden
     * con los parámetros.
     */

    public boolean eliminarDatosDispositivo(String Device){
        SQLiteDatabase db = baseDatos.getWritableDatabase();

        String selection = String.format("%s=?",
                DeviceContract.DatosDispositivos.MDEVICE);
        String[] whereArgs = {Device};

        int resultado = db.delete(Tablas.DATOS_DISPOSITIVO, selection, whereArgs);

        return resultado > 0;
    }

    /** El siguiente método comprueba si el dispositivo existe y devuelve true o false
     * según el resultado
     * @param Device
     * @return boolean
     */

    public boolean ifDeviceExists(String Device){
        SQLiteDatabase db = baseDatos.getReadableDatabase();

        String selection = String.format("%s=?", DeviceContract.DatosDispositivos.MDEVICE);

        if (selection.equals(Device)) return true;
        else return false;
    }

    /**
     * [/OPERACIONES_DATOS_DISPOSITIVO]
     * [OPERACIONES_USUARIO]
     * El siguiente método obtendrá todos los datos de Usuarios
     */

    public Cursor obtenerUsuarios(){
        SQLiteDatabase db = baseDatos.getReadableDatabase();

        SQLiteQueryBuilder builder = new SQLiteQueryBuilder();

        builder.setTables(DATOS_DISPOSITIVO_JOIN_USUARIO);
        return builder.query(db, proyUsuarios, null, null, null, null, null);
    }

    /**
     * El siguiente método obtendrá los datos de un Usuario especificado por una ID
     */

    public Cursor obtenerUsuarioPorId(String idUsuario){
        SQLiteDatabase db = baseDatos.getReadableDatabase();

        String selection = String.format("%s=?", DeviceContract.Usuarios.ID);
        String[] selectionArgs = {idUsuario};

       SQLiteQueryBuilder builder = new SQLiteQueryBuilder();
        builder.setTables(DATOS_DISPOSITIVO_JOIN_USUARIO);

        return builder.query(db, proyUsuarios, selection, selectionArgs, null, null, null);
    }

    /**
     * El siguiente método insertará un nuevo Usuario mediante una instancia Usuario.
     * Como resultado devolverá una ID de Usuario.
     */

    public String insertarUsuario(Usuario usuario){
        SQLiteDatabase db = baseDatos.getWritableDatabase();
        String idUsuario = DeviceContract.Usuarios.generarIdCoUsuarios();
        ContentValues values = new ContentValues();
        //Generar pk
        if (usuario.idUsuario == null || usuario.idUsuario.equals("")) {
            values.put(DeviceContract.Usuarios.ID, idUsuario);
        }else{
            values.put(DeviceContract.Usuarios.ID, usuario.idUsuario);
        }
        values.put(DeviceContract.Usuarios.USUARIO_DEVICE, usuario.UsuarioDevice);
        values.put(DeviceContract.Usuarios.NOMBRE, usuario.nombre);
        values.put(DeviceContract.Usuarios.ALTURA, usuario.altura);
        values.put(DeviceContract.Usuarios.PESO, usuario.peso);
        values.put(DeviceContract.Usuarios.ZANCADA, usuario.zancada);
        values.put(DeviceContract.Usuarios.LASTLOG, usuario.lastlog);
        values.put(DeviceContract.Usuarios.RETODIARIO, usuario.RetoDiario);
        values.put(DeviceContract.Usuarios.RETOCOMPLTADO, usuario.RetoCompleatado);

        db.insertOrThrow(Tablas.DATOS_USUARIO, null, values);
        return idUsuario;
    }

    /**
     * Este método actualizará los datos de un usuario que coincidan en ID
     */

    public boolean actualizarUsuario(Usuario usuario){
        SQLiteDatabase db = baseDatos.getWritableDatabase();

        ContentValues values = new ContentValues();
        //values.put(DeviceContract.Usuarios.NOMBRE, usuario.nombre);
        values.put(DeviceContract.Usuarios.ALTURA, usuario.altura);
        values.put(DeviceContract.Usuarios.PESO, usuario.peso);

        String whereClause = String.format("%s=?", DeviceContract.Usuarios.ID);
        String[] whereArgs = {usuario.idUsuario};

        int resultado = db.update(Tablas.DATOS_USUARIO, values, whereClause, whereArgs);

        return resultado > 0;
    }

    /**
     * Este método actualizará el lastlog de un usuario que coincida en ID a traves de una instancia
     * Usuariio
     */

    public boolean actualizarLastLog(Usuario usuario){
        SQLiteDatabase db = baseDatos.getWritableDatabase();

        ContentValues values = new ContentValues();
        values.put(DeviceContract.Usuarios.LASTLOG, usuario.lastlog);

        String whereClause = String.format("%s=?", DeviceContract.Usuarios.ID);
        String[] whereArgs = {usuario.idUsuario};

        int resultado = db.update(Tablas.DATOS_USUARIO, values, whereClause, whereArgs);

        return resultado > 0;
    }

    /**
     *  Este método actualiza el reto diario a hacer el usuario mediante una instancia de Usuario
     * @param usuario
     * @return
     */

    public boolean actualizarRetoDiario(Usuario usuario){
        SQLiteDatabase db = baseDatos.getWritableDatabase();

        ContentValues values = new ContentValues();
        values.put(DeviceContract.Usuarios.RETODIARIO, usuario.RetoDiario);

        String whereClause = String.format("%s=?", DeviceContract.Usuarios.ID);
        String[] whereArgs = {usuario.idUsuario};

        int resultado = db.update(Tablas.DATOS_USUARIO, values, whereClause, whereArgs);

        return resultado > 0;
    }

    public boolean actualizarRetoCompletado(Usuario usuario){
        SQLiteDatabase db = baseDatos.getWritableDatabase();

        ContentValues values = new ContentValues();
        values.put(DeviceContract.Usuarios.RETOCOMPLTADO, usuario.RetoCompleatado);

        String whereClause = String.format("%s=?", DeviceContract.Usuarios.ID);
        String[] whereArgs = {usuario.idUsuario};

        int resultado = db.update(Tablas.DATOS_USUARIO, values, whereClause, whereArgs);

        return resultado > 0;
    }

    /**
     * El siguiente método actualiza los pasos que ha dado el usuario mal.
     * Solo lo hace si la ID del usuario coincide.
     */

    public boolean actualizarZancada(String id, String zancada){
        SQLiteDatabase db = baseDatos.getWritableDatabase();

        ContentValues values = new ContentValues();
        values.put(DeviceContract.Usuarios.ZANCADA, zancada);

        String whereClause = String.format("%s=?", DeviceContract.Usuarios.ID);
        String[] whereArgs = {id};

        int resultado = db.update(Tablas.DATOS_USUARIO, values, whereClause, whereArgs);

        return resultado > 0;
    }

    /**
     * Este método eliminará el Usuario que coincida con el ID pasado
     */

    public boolean eliminarUsuario(String idUsuario){
        SQLiteDatabase db = baseDatos.getWritableDatabase();

        String whereClause = String.format("%s=?", DeviceContract.Usuarios.ID);
        String[] whereArgs = {idUsuario};

        int resultado = db.delete(Tablas.DATOS_USUARIO, whereClause, whereArgs);

        return resultado > 0;
    }

    /**
     * El siguiente método eliminará un Usuario por el nombre
     */

    public boolean eliminarUsuarioporNombre(String Nombre){
        SQLiteDatabase db = baseDatos.getWritableDatabase();

        String whereClause = String.format("%s=?", DeviceContract.Usuarios.NOMBRE);
        String[] whereArgs = {Nombre};

        int resultado = db.delete(Tablas.DATOS_USUARIO, whereClause, whereArgs);

        return resultado > 0;
    }

    /**
     * [/OPERACIONES_USUARIO]
     * [OPERACIONES RENDIMIENTO USUARIO]
     * El siguiente método obtendrá todos los rendimientos de Usuario     *
     */

    public Cursor obtenerRendimientosUsuario(){
        SQLiteDatabase db = baseDatos.getReadableDatabase();

        SQLiteQueryBuilder builder = new SQLiteQueryBuilder();

        builder.setTables(USUARIO_JOIN_RENDIMIENTO_USUARIO);
        return builder.query(db, proyRendimientoUsuario, null, null, null, null, null);
    }

    /**
     * El siguiente método obtendrá los datos de un Rendimiento especificado por una ID
     */

    public Cursor obtenerRendimientoPorFecha(String Fecha){
        SQLiteDatabase db = baseDatos.getReadableDatabase();

        String selection = String.format("%s=?", DeviceContract.RendimientoUsuarios.FECHA);
        String[] selectionArgs = {Fecha};

        SQLiteQueryBuilder builder = new SQLiteQueryBuilder();
        builder.setTables(USUARIO_JOIN_RENDIMIENTO_USUARIO);

        return builder.query(db, proyRendimientoUsuario, selection, selectionArgs, null, null, null);
    }

    /**
     * El siguiente método insertará un nuevo Rendimiento mediante una instancia Rendimiento.
     * Como resultado devolverá una ID de Usuario.
     */

    public String insertarRendimiento(RendimientoUsuario rendimientoUsuario){
        SQLiteDatabase db = baseDatos.getWritableDatabase();
        String idRendimiento = DeviceContract.RendimientoUsuarios.generarIdRenUsuario();
        ContentValues values = new ContentValues();
        //Generar pk
        values.put(DeviceContract.Usuarios.ID, idRendimiento);

        values.put(DeviceContract.RendimientoUsuarios.NOMBRE, rendimientoUsuario.mNombre);
        values.put(DeviceContract.RendimientoUsuarios.FECHA, rendimientoUsuario.mDate);
        values.put(DeviceContract.RendimientoUsuarios.PASOS, rendimientoUsuario.mSteps);
        values.put(DeviceContract.RendimientoUsuarios.PASOS_MALOS, rendimientoUsuario.mBadSteps);
        values.put(DeviceContract.RendimientoUsuarios.PASOS_MALOS_2, rendimientoUsuario.mBadSteps2);
        values.put(DeviceContract.RendimientoUsuarios.PASOS_PESADOS, rendimientoUsuario.mHeavySteps);
        values.put(DeviceContract.RendimientoUsuarios.DISTANCIA, rendimientoUsuario.mDistance);
        values.put(DeviceContract.RendimientoUsuarios.CALORIAS_QUEMADAS, rendimientoUsuario.mBurnedCalories);

        db.insertOrThrow(Tablas.DATOS_RENDIMIENTO_USUARIO, null, values);
        return idRendimiento;
    }


    /**
     * Este método eliminará el Rendimiento que coincida con la fecha demandada
     */

    public boolean eliminarRendimiento(String Fecha){
        SQLiteDatabase db = baseDatos.getWritableDatabase();

        String whereClause = String.format("%s=?", DeviceContract.RendimientoUsuarios.FECHA);
        String[] whereArgs = {Fecha};

        int resultado = db.delete(Tablas.DATOS_RENDIMIENTO_USUARIO, whereClause, whereArgs);

        return resultado > 0;
    }

    public SQLiteDatabase getDb() {
        return baseDatos.getWritableDatabase();
    }
}
